"""Riemann TCP and UDP transports"""

from __future__ import absolute_import

import abc
import os
import socket
import struct

import riemann.riemann_pb2


class RiemannError(Exception):
    """Error class for errors recived from the Riemann server"""
    pass


class Transport(object):
    __metaclass__ = abc.ABCMeta

    def __init__(self, host=None, port=None):
        self.host = host or os.environ.get('RIEMANN_HOST', 'localhost')
        self.port = port or os.environ.get('RIEMANN_PORT', 5555)

    def __enter__(self):
        self.connect()
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        self.disconnect()

    @abc.abstractmethod
    def connect(self):
        pass

    @abc.abstractmethod
    def disconnect(self):
        pass

    @abc.abstractmethod
    def write(self):
        pass

    @property
    def address(self):
        return self.host, self.port


class UDPTransport(Transport):
    def connect(self):
        self.socket = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)

    def disconnect(self):
        self.socket.close()

    def write(self, message):
        self.socket.sendto(message.SerializeToString(), self.address)


class TCPTransport(Transport):
    def connect(self):
        self.socket = socket.create_connection(self.address)

    def disconnect(self):
        self.socket.close()

    def write(self, message):
        # Send the message to the server
        data = message.SerializeToString()
        self.socket.send(struct.pack('!I', len(data)) + data)

        # Return the server's response
        response_len = struct.unpack('!I', self.socket.recv(4))[0]
        response = riemann.riemann_pb2.Msg()
        response.ParseFromString(self.socket.recv(response_len))

        # Handle error messages
        if not response.ok:
            raise RiemannError(response.error)

        return response



#!/usr/bin/env python2.6

import setuptools
import sys

setuptools.setup(
    name = "riemann-client",
    version = '2.0.1',

    author = "Sam Clements",
    author_email = "sam.clements@datasift.com",

    url = "https://github.com/borntyping/python-riemann-client",
    description = "A Riemann client and command line tool",
    long_description = open('README.rst').read(),

    packages = setuptools.find_packages(),

    install_requires = [
        'argparse==1.2.1',
        'protobuf==2.5.0',
    ],

    entry_points = {
        'console_scripts': [
            'riemann-client = riemann.command:main',
            'riemann-client-{0}.{1} = riemann.command:main'.format(*sys.version_info)
        ]
    },

    classifiers = [
        'Development Status :: 4 - Beta',
        'License :: OSI Approved',
        'License :: OSI Approved :: MIT License',
        'Operating System :: Unix',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Topic :: Software Development :: Libraries',
        'Topic :: System :: Monitoring',
        'Topic :: System :: Networking',
        'Topic :: System :: Systems Administration'
    ],
)
